# spline-based ANCOVA test to identify section-dependent genes
section.test.splines.f = function(Mat, x, DEGREE=8, VERBOSE=F) {
	# test smooth spline models with different degrees of freedom (2:8) and get the one with the lowest F-test p-value
	# Mat: expression matrix (row: genes; column: samples)
	# x: section index, with the same length as the number of columns in Mat
	# DEGREE: the maximum degree of freedom for spline
	
	Res <- t(sapply(1:nrow(Mat), function(i) {
		if (VERBOSE == TRUE) print(i)
		y <- as.numeric(Mat[i, ])
		null <- lm(y ~ 1)
		res1 <- ifelse(sum(null$resi^2)<10^(-10), 0, sum(null$resi^2))
		df1 <- null$df
		res2l <- sapply(2:DEGREE, function(i){ sum((predict(smooth.spline(x, y, df=i), x)$y - y)^2) })
		dfl <- df1 - 1:(DEGREE-1)
		pvals <- sapply(1:(DEGREE-1), function(i) {
			res2 <- ifelse(res2l[[i]]<10^(-10), 0, res2l[[i]])
			df2 <- dfl[[i]]
			fstat <- ((res1 - res2)/(df1 - df2))/(res2/df2)
			pf(q = fstat, df1 = df1-df2, df2 = df2, lower.tail = F)
		})
		if(is.nan(max(pvals))){
			return(c(NA,1))
		} else{
			return(c(which.min(pvals)+1, pvals[which.min(pvals)]))
		}
	}))
	rownames(Res) = rownames(Mat)
	colnames(Res) = c("Model", "P-value")
	return(Res)
}

# spline-based ANCOVA test to identify genes with expression hetero-spatiality in two species 
species.test.spline.f <- function(expr, species, sections, num.permut=1000){
	# test whether smooth spline for two species separately is significantly better than pooled smooth spline
	# expr: expression matrix (row: genes, columns: samples)
	# species: species index, with the same length as the number of columns in expr
	# sections: section index, with the same length as the number of columns in expr
	# num.permut: the number of permutations used to estimate permutation-based p-value
	
	idx1 <- which(species==unique(species)[1])
	idx2 <- which(species==unique(species)[2])
	permut.species <- sapply(1:num.permut, function(i){ sample(species) })
	
	res <- apply(expr, 1, function(x){
		if(sum(is.na(x))>0){
			return(c(NA,NA,NA))
		}
		m0 <- smooth.spline(x=sections, y=x)
		m1.1 <- smooth.spline(x=sections[idx1], y=x[idx1])
		m1.2 <- smooth.spline(x=sections[idx2], y=x[idx2])
		res1 <- sum((x-predict(m0, sections)$y)^2)
		res2 <- sum((x[idx1]-predict(m1.1, sections[idx1])$y)^2) + sum((x[idx2]-predict(m1.2, sections[idx2])$y)^2)
		fstat <- (res1-res2)/res2
		
		fstat.permut <- sapply(1:num.permut, function(i){
			idx1.permut <- which(permut.species[,i]==unique(species)[1])
			idx2.permut <- which(permut.species[,i]==unique(species)[2])
			
			res2 <- sum((x[idx1.permut]-predict(smooth.spline(x=sections[idx1.permut], y=x[idx1.permut]), sections[idx1.permut])$y)^2) + sum((x[idx2.permut]-predict(smooth.spline(x=sections[idx2.permut], y=x[idx2.permut]), sections[idx2.permut])$y)^2)
			return((res1-res2)/res2)
		})
		
		p.fdistrib <- 1-pf(fstat*(length(x)-(m1.1$df+m1.2$df)-1)/(m1.1$df+m1.2$df-m0$df), m1.1$df+m1.2$df-m0$df, length(x)-(m1.1$df+m1.2$df)-1)
		res <- ifelse(sum(fstat.permut>fstat)==0, 1/length(fstat.permut), sum(fstat.permut>fstat)/length(fstat.permut))
		return(c(fstat, res, p.fdistrib))
	})
	
	return(t(res))
}

